/******************************************************
*  File: mouse-keyboard.c
*  Purpose:  This file updates data for keyboard and
*            mouse activities
*
*  Author     Date      Notes
*  morelli    04/03/04  Initial Version
*
******************************************************/



#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include "/usr/include/mysql/mysql.h"




int CheckForInputs(unsigned long * _mouseLoc, unsigned long * _keyboardLoc);
int GetMinutes();
int UpdateDB(int mouseChange, int keyboardChange);
int SumData();
int GetRandom(int low, int hi);


int main()
{
  unsigned long mouseLoc, keyboardLoc, oldMouseLoc, oldKeyboardLoc;
  int change;
  int minutes;
  mouseLoc = 0;
  keyboardLoc = 0;
  SumData();
  while(1)
  {
    minutes = GetMinutes();
    if ((minutes == 0) || (minutes == 5))
    {
      oldMouseLoc = mouseLoc;
      oldKeyboardLoc = keyboardLoc;
      change = CheckForInputs(&mouseLoc, &keyboardLoc);
      if ((oldMouseLoc !=0) || (oldKeyboardLoc != 0))
      {
        printf("MouseChange: %ld\n", mouseLoc - oldMouseLoc);
        printf("KeyboardChange: %ld\n", keyboardLoc - oldKeyboardLoc);
        UpdateDB(mouseLoc - oldMouseLoc, keyboardLoc - oldKeyboardLoc);

      }
    }
    else if ((minutes == 1) || (minutes == 6))
    {
      SumData();
    }
    sleep(60);
  }
}



int CheckForInputs(unsigned long * _mouseLoc, unsigned long * _keyboardLoc)
{
  FILE* fp;
  char buf[256];
  unsigned long oldMouse, oldKeyboard;
  oldMouse = *_mouseLoc;
  oldKeyboard = *_keyboardLoc;

  system("expect poll");

  system("cat ./interrupts | grep keyboard > keyboard.out");
  system("cat ./interrupts | grep Mouse > mouse.out");

  fp = fopen("mouse.out", "r");
  fscanf(fp, "%s", buf);
  fscanf(fp, "%ld", _mouseLoc);
  fclose(fp);

  fp = fopen("keyboard.out", "r");
  fscanf(fp, "%s", buf);
  fscanf(fp, "%ld", _keyboardLoc);
  fclose(fp);

  if ((*_mouseLoc != oldMouse) || (*_keyboardLoc != oldKeyboard))
    return 1;
  else
    return 0;

}

int GetMinutes()
{
  time_t ttime;
  char * strTime;
  char temp[2];
  int minutes;
  strTime = (char *) malloc (30 * sizeof (char));
  time(&ttime);
  strTime = ctime(&ttime);
  temp[0] = strTime[15];
  temp[1] = '\0';
  minutes = atoi(temp);
  return minutes;
}

int UpdateDB(int mouseChange, int keyboardChange)
{
  MYSQL mysql;
  MYSQL_RES *result;
  MYSQL_ROW row;
  const char * sql_insert = "insert into numbers (iIndex) values (%d)";
  char insert[4096];
  int x;

 if(!mysql_init(&mysql))
   {
      fprintf(stderr, "Cannot initialize MySQL");
      return(1);
   }

 if(!mysql_real_connect(&mysql, "localhost",
      "root", "", "emotion", 0, NULL, 0))
   {
      fprintf(stderr, "%d: %s \n",
         mysql_errno(&mysql), mysql_error(&mysql));
      return(1);
   }
   sprintf(insert, "INSERT into emotion.tempstats (dtEntered, iSensorID, iSensorVal) VALUES (NOW(), 3, %d)", keyboardChange);
   mysql_query(&mysql, insert); //checkForErrors...
   sprintf(insert, "INSERT into emotion.tempstats (dtEntered, iSensorID, iSensorVal) VALUES (NOW(), 4, %d)", mouseChange);
   mysql_query(&mysql, insert); //checkForErrors...

   mysql_close(&mysql);
  
   return 0;

}
int  SumData()
{

  MYSQL mysql;
  MYSQL_RES *result;
  MYSQL_ROW row;
  const char * sql_insert = "insert into numbers (iIndex) values (%d)";
  char insert[4096];
  int sensorVals[5];
  int x, nextPictureType, nextPictureId;
  
 if(!mysql_init(&mysql))
   {
      fprintf(stderr, "Cannot initialize MySQL");
      return(1);
   }

 if(!mysql_real_connect(&mysql, "localhost",
      "root", "", "emotion", 0, NULL, 0))
   {
      fprintf(stderr, "%d: %s \n",
         mysql_errno(&mysql), mysql_error(&mysql));
      return(1);
   }
  for (x = 1; x<5; x++)
  {
    sprintf(insert, "SELECT iSensorVal FROM emotion.tempstats WHERE iSensorID = %d ORDER BY iTempStatID DESC LIMIT 1", x);
    if (!mysql_query(&mysql, insert))
    {
      result = mysql_store_result(&mysql);
      while (row = mysql_fetch_row(result))
      {
        sensorVals[x] = atoi(row[0]);
      }
      mysql_free_result(result);
    }
  }

  nextPictureType = GetRandom(0,1); 

  sprintf(insert, "SELECT iPictureId from emotion.pictures where iMoodId = %d", nextPictureType);
  if (!mysql_query(&mysql, insert))
  {
    result = mysql_store_result(&mysql);
    printf("%ld\n", mysql_num_rows(result));
    mysql_data_seek(result, GetRandom(1,mysql_num_rows(result))-1);
    row = mysql_fetch_row(result);
    nextPictureId = atoi(row[0]);
    mysql_free_result(result);
  } 


  printf("%d %d %d\n", sensorVals[1], sensorVals[3], sensorVals[4]);

  sprintf(insert, "INSERT INTO emotion.stats (iPictureId, dtTimeShown, iVideoVal, iKeyBoardVal, iMouseVal) VALUES (%d, NOW(), %d, %d, %d)", nextPictureId, sensorVals[1], sensorVals[3], sensorVals[4]);

//  sprintf(insert, "INSERT INTO emotion.stats (dtTimeShown, iKeyBoardVal, iMouseVal) VALUES (%d, NOW(), %d, %d)", sensorVals[1], sensorVals[3], sensorVals[4]);
  mysql_query(&mysql, insert); //checkForErrors...

  mysql_close(&mysql);
  return 0;
}

int GetRandom(int low, int hi)
{
  return (random() % (hi - low) + low);
}

